jQuery.validator.methods.notempty = function (value, element) {
  return value.trim().length !== 0;
};

jQuery.validator.addMethod('completeCredentials', function (value, element, params) {
  var username = $(element).parents('form').find('input[name="' + params[0] + '"]'),
    password = $(element).parents('form').find('input[name="' + params[1] + '"]');

  if ($.validator.methods.notempty(username.val(), username)) {
    return $.validator.methods.notempty(password.val(), password);
  } else if ($.validator.methods.notempty(password.val(), password)) {
    return $.validator.methods.notempty(username.val(), username);
  }
  return true
}, 'You must supply both a username and password.');

$.validator.addMethod('moreThanZero', function (value) {
  return value > 0;
}, 'The size must be an integer > 0.');

var validation = {
  'registration-form': {
    rules: {
      controllerAddress: {
        required: true,
        notempty: true
      },
      address: {
        required: true,
        notempty: true
      },
      key: {
        required: true,
        notempty: true
      }
    }
  },
  purge: {},
  targetCacheSize: {
    rules: {
      targetCacheSize: {
        required: true,
        min: 1
      }
    }
  },
  maxArtifactSize: {
    rules: {
      maxArtifactSize: {
        required: true,
        min: 1
      }
    }
  },
  credentials: {
    rules: {
      primaryUsername: {
        completeCredentials: ['primaryUsername', 'primaryPassword']
      },
      primaryPassword: {
        completeCredentials: ['primaryUsername', 'primaryPassword']
      },
      alternateUsername: {
        completeCredentials: ['alternateUsername', 'alternatePassword']
      },
      alternatePassword: {
        completeCredentials: ['alternateUsername', 'alternatePassword']
      }
    },
    groups: {
      primary: 'primaryUsername primaryPassword',
      alternate: 'alternateUsername alternatePassword'
    }
  }
};

var app = function () {
  var bindValidation = function () {
    $('form').each(function () {
      var form = $(this);
      $(this).validate({
        ignore: '',
        onfocusout: false,
        onkeyup: false,
        errorPlacement: function (error, element) {
        },
        showErrors: function (errorMap, errorList) {
          var arrayLength = errorList.length;
          for (var i = 0; i < arrayLength; i++) {
            if (i === 0) {
              errorMsg(errorList[i].message, form.find('.msg'));
            }
            $(errorList[i].element).addClass('error');
          }
          this.defaultShowErrors();
        },
        highlight: function (element, errorClass) {
          $(element).addClass(errorClass);
        },
        unhighlight: function (element, errorClass) {
          $(element).removeClass(errorClass);
        },
        rules: validation[$(this).attr('id')].rules,
        groups: validation[$(this).attr('id')].groups,
        submitHandler: function (form, event) {
          event.preventDefault();
          var $form = $(form)
          jQuery.post($form.attr('action'), $form.serialize())
            .done(function () {
              if ($form.attr('id') === 'registration-form') {
                $('input[name=secret]').attr('placeholder', '(hidden)').val('');
              }
              successMsg($form.data('successMsg'), $form.find('.msg'));
              var callback = $form.data('onSubmit');
              if (callback) {
                callback();
              }
            })
            .fail(function () {
              errorMsg($form.data('failureMsg'), $form.find('.msg'));
            });
        }
      });
    });
  };

  jQuery(function () {
    bindAccordion();
    updateUi();
    fixFocus();
    fixRefreshForTabbing();
    bindValidation();
  });

  var updateTime = Date.now();

  var fixRefreshForTabbing = function () {
    // If a user is tabbing between elements, back off the refreshing to allow them to "click" submit
    $('input[type=text]').on('blur', function () {
      updateTime = Date.now();
    });
  };

  var updateConnectionStatus = function () {
    return jQuery.get('/status')
      .done(function (data) {
        var status = jQuery('#status');
        var html = status.html();
        if (html !== data.message) {
          status.removeClass().addClass(data.error ? 'error' : 'success');
          status.html(data.message);
        }
      })
      .fail(function (e) {
        console.log('failed to update status', e);
      });
  };

  var updateUi = function () {
    // Only update the client UI if it hasn't been touched for 4s
    if (Date.now() - updateTime > 5000) {
      jQuery.get('/snapshot')
        .done(function (data) {
          Object.keys(data).forEach(function (key) {
            var value = data[key];
            var element = $('.refreshable.' + key);
            var updated = false;
            if (element.is('input') && element.val() !== '' + value && !element.is(':focus')) {
              element.val(value);
              updated = true;
            } else if (element.text() !== '' + value) {
              if (key === 'formattedSizeSnapshot') {
                if (value === $('#cache-state').data('empty-msg')) {
                  $('form#purge input').attr('disabled', 'yes');
                }
                else {
                  $('form#purge input').removeAttr('disabled');
                }
              }
              element.text(value);
              updated = true;
            }
            if (updated) {
              element.fadeTo(50, 0.3, function () {
                $(this).fadeTo(200, 1.0);
              });
            }
          });
        })
        .fail(function (e) {
          console.log('failed to refresh from snapshot', e);
        })
    }

    updateConnectionStatus()
      .always(function () {
        setTimeout(updateUi, 2000)
      });
  };

  $("form#registration-form").data("onSubmit", function() {
    $("#status").html("<span class='connecting'>Connecting…");
  });

  var fixFocus = function () {
    // If the user is tabbing around, we want to show the focus ring
    document.addEventListener('keydown', function (e) {
      if (e.keyCode === 9) {
        $('body').addClass('show-focus-outlines');
      }
    });
    // But if they're clicking about, we don't
    document.addEventListener('click', function (e) {
      $('body').removeClass('show-focus-outlines');
    });
  };

  var bindAccordion = function () {
    var clickHandler = function (e) {
      if (e.type === 'keydown' && e.key !== ' ') {
        return;
      }
      // collapse all first
      $('section.accordion').each(function () {
        var acc = $(this);
        if (!acc.hasClass('collapsed')) {
          acc.find('.content').slideUp(function () {
            acc.addClass('collapsed');
          });
        }
      });

      var content = $(this).parent().find('.content');
      content.parent().removeClass('expansion-done');
      if (content.parent().hasClass('collapsed')) {
        content.slideDown(300, function () {
          content.parent().addClass('expansion-done');
        });
        content.parent().removeClass('collapsed');
      } else {
        content.slideUp(300, function () {
          content.parent().addClass('expansion-done');
        });
        content.parent().addClass('collapsed');
      }
    };

    $('section.accordion > .accordion-toggle:first-child').on('click keydown', clickHandler);
  };
};

function ajax(url, method, payload) {
  return jQuery.ajax({
    method: method,
    url: url,
    data: payload,
    beforeSend: function(request) {
      request.setRequestHeader("X-Gradle-Enterprise-Ajax-Request", "1");
    }
  });
}